/* Next Generation Common API
   Copyright  2003-2004 by Next Generation.

   Permission is hereby granted, free of charge, to any person
   obtaining a copy of this software and associated documentation files
   (the "Software"), to deal in the Software without restriction,
   including without limitation the rights to use, copy, modify, merge,
   publish, distribute, sublicense, and/or sell copies of the Software,
   and to permit persons to whom the Software is furnished to do so,
   subject to the following conditions:

   The above copyright notice and this permission notice shall be
   included in all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
   NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
   BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
   ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
   SOFTWARE.
   
   ---

   Please send comments, bugreports and patches to <chris@ngweb.biz>.
   Visit this website for more information: http://www.ngweb.biz/
 */

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <stdlib.h>
#include <string.h>
#include <tchar.h>
#include "iofunc.h"

DWORD
#ifdef UNICODE
GetRegistryValueW
#else
GetRegistryValueA
#endif /* UNICODE */
(HKEY hKey, LPCTSTR lpSubKey, LPCTSTR lpValueName, LPVOID lpDefaultValue, LPVOID lpReturnValue, DWORD dwSize, int nType)
{
  BOOL bCloseKey = FALSE, bSuccess = TRUE;
  DWORD dwRealSize;
  HKEY hkey;
  LPBYTE lpByte;
  
  if (RegOpenKeyEx (hKey, lpSubKey, 0, KEY_ALL_ACCESS, &hkey) != ERROR_SUCCESS)
    bSuccess = FALSE;
  else
    bCloseKey = TRUE;

  if (bSuccess)
    {
      if (RegQueryValueEx (hkey, lpValueName, NULL, NULL, NULL, &dwRealSize) != ERROR_SUCCESS)
        bSuccess = FALSE;
    }
    
  if (!bSuccess)
    {
      if (lpDefaultValue)
        {
          switch (nType)
            {
            case GRVT_DWORD:
              dwRealSize = 0;
              CopyMemory (lpReturnValue, lpDefaultValue, sizeof (DWORD));
              break;
              
            case GRVT_STRING:
            case GRVT_EXPAND:
              dwRealSize = _tcslen ((LPCTSTR) lpDefaultValue);
              if (dwSize < dwRealSize)
                break;
              CopyMemory (lpReturnValue, lpDefaultValue, dwSize * sizeof (TCHAR));
              break;
            }
        }
      else
        dwRealSize = 0;
      goto theEnd;
    }
  
  if ((dwSize < dwRealSize || !lpReturnValue) && nType == GRVT_STRING)
    goto theEnd;

  lpByte = (LPBYTE) malloc (sizeof (LPBYTE) * dwRealSize);
  RegQueryValueEx (hkey, lpValueName, NULL, NULL, lpByte, &dwRealSize);
  CopyMemory (lpReturnValue, lpByte, dwRealSize);
  free (lpByte);

theEnd:
  if (bCloseKey)
    RegCloseKey (hkey);
  return dwRealSize;
}
